@@ -41,7 +41,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#include <vector>
+#include <list>
 
 #ifdef OSX_CORE_AUDIO
 #include "../MacOSX/MacOSX_sound_if.h"
@@ -49,7 +49,7 @@ static int bincue_core_audio_callback(void);
 #endif
 
 #ifdef USE_SDL_AUDIO
-#include <SDL.h>
+#include "my_sdl.h"
 #include <SDL_audio.h>
 #endif
 
@@ -111,6 +111,7 @@ typedef struct {
 	int raw_sector_size;	// Raw bytes to read per sector
 	int cooked_sector_size; // Actual data bytes per sector (depends on Mode)
 	int header_size;		// Number of bytes used in header
+	int big_endian_audio;   // Expect raw audio samples in big-endian format
 } CueSheet;
 
 typedef struct CDPlayer {
@@ -125,7 +126,9 @@ typedef struct CDPlayer {
 	uint8 volume_right;			// CD player volume (right)
 	uint8 volume_mono;			// CD player single-channel volume
 	loff_t fileoffset;			// offset from file beginning to audiostart
-	bool audio_enabled = false; // audio initialized for this player?
+	bool audio_enabled;			// audio initialized for this player?
+	bool scanning;				// is there currently scanning in progress
+	int reverse;                // for scanning, 0=forward, 1=reverse
 #ifdef OSX_CORE_AUDIO
 	OSXsoundOutput soundoutput;
 #endif
@@ -145,20 +148,46 @@ typedef struct {
 static unsigned int totalPregap;
 static unsigned int prestart;
 
+// Current audio output settings
+
+struct OutputSettings {
+	int freq;
+	int format; // SDL format
+	int channels;
+	int default_cd_player_volume;
+};
+
+static bool have_current_output_settings = false;
+static OutputSettings current_output_settings;
+
 // Audio System Variables
 
 static uint8 silence_byte;
 
+// CD Player state; multiple players supported through list
 
-// CD Player state; multiple players supported through vectors
-
-std::vector<CDPlayer*> players;
-
+static std::list<CDPlayer*> players;
 CDPlayer* currently_playing = NULL;
 
+#if SDL_VERSION_ATLEAST(3, 0, 0)
+static SDL_Mutex *player_lock;
+#else
+static SDL_mutex *player_lock;
+#endif
+#define LOCK_PLAYER		SDL_LockMutex(player_lock)
+#define UNLOCK_PLAYER	SDL_UnlockMutex(player_lock)
+
+void InitBinCue() {
+	player_lock = SDL_CreateMutex();
+}
+
+void ExitBinCue() {
+	SDL_DestroyMutex(player_lock);
+}
+
 CDPlayer* CSToPlayer(CueSheet* cs)
 {
-	for (std::vector<CDPlayer*>::iterator it = players.begin(); it != players.end(); ++it)
+	for (std::list<CDPlayer*>::iterator it = players.begin(); it != players.end(); ++it)
 		if (cs == (*it)->cs) // look for cuesheet matching existing player
 			return *it;
 	return NULL; // if no player with the cuesheet found, return null player
@@ -268,6 +297,7 @@ static bool ParseCueSheet(FILE *fh, CueSheet *cs, const char *cuefile)
 	cs->raw_sector_size = 2352;
 	cs->cooked_sector_size = 2352;
 	cs->header_size = 0;
+	cs->big_endian_audio = false;
 
 	while (fgets(line, MAXLINE, fh) != NULL) {
 		Track *curr = &cs->tracks[cs->tcnt];
@@ -292,11 +322,13 @@ static bool ParseCueSheet(FILE *fh, CueSheet *cs, const char *cuefile)
 				}	
 				filename = strtok(NULL, "\"\t\n\r");
 				filetype = strtok(NULL, " \"\t\n\r");
-				if (strcmp("BINARY", filetype)) {
-					D(bug("Not binary file %s", filetype));
+				if (strcmp("BINARY", filetype) && strcmp("MOTOROLA", filetype)) {
+					D(bug("Not binary file %s\n", filetype));
 					goto fail;
 				}
 				else {
+					if (!strcmp("MOTOROLA", filetype))
+						cs->big_endian_audio = true;
 					char *tmp = strdup(cuefile);
 					char *b = dirname(tmp);
 					cs->binfile = (char *) malloc(strlen(b) + strlen(filename) + 2);
@@ -478,6 +510,10 @@ static bool LoadCueSheet(const char *cuefile, CueSheet *cs)
 	return false;
 }
 
+#ifdef USE_SDL_AUDIO
+	static void OpenPlayerStream(CDPlayer * player);
+	static void ClosePlayerStream(CDPlayer * player);
+#endif
 
 
 void *open_bincue(const char *name)
@@ -487,13 +523,14 @@ void *open_bincue(const char *name)
 		D(bug("malloc failed\n"));
 		return NULL;
 	}
-
 	if (LoadCueSheet(name, cs)) {
 		CDPlayer *player = (CDPlayer *) malloc(sizeof(CDPlayer));
 		player->cs = cs;
 		player->volume_left = 0;
 		player->volume_right = 0;
 		player->volume_mono = 0;
+		player->audio_enabled = false;
+		player->scanning = false;
 #ifdef OSX_CORE_AUDIO
 		player->audio_enabled = true;
 #endif
@@ -502,10 +539,14 @@ void *open_bincue(const char *name)
 		else
 			player->audiostatus = CDROM_AUDIO_INVALID;
 		player->audiofh = dup(cs->binfh);
-		
+
+#ifdef USE_SDL_AUDIO
+		OpenPlayerStream(player);
+#endif
+
 		// add to list of available CD players
 		players.push_back(player);
-		
+
 		return cs;
 	}
 	else
@@ -518,15 +559,18 @@ void close_bincue(void *fh)
 {
 	CueSheet *cs = (CueSheet *) fh;
 	CDPlayer *player = CSToPlayer(cs);
-	
+
 	if (cs && player) {
+		if (player == currently_playing) {
+			CDStop_bincue(fh);
+			assert(currently_playing == NULL);
+		}
+
+		players.remove(player);
+
 		free(cs);
 #ifdef USE_SDL_AUDIO
-#if !SDL_VERSION_ATLEAST(3, 0, 0)
-#define SDL_DestroyAudioStream	SDL_FreeAudioStream
-#endif
-		if (player->stream) // if audiostream has been opened, free it as well
-			SDL_DestroyAudioStream(player->stream);
+		ClosePlayerStream(player);
 #endif
 		free(player);
 	}
@@ -713,6 +757,7 @@ bool CDPause_bincue(void *fh)
 		// Pause another player if needed
 		CDPause_playing(player);
 
+		player->scanning = false;
 		// doesn't matter if it was playing, just ensure it's now paused
 		player->audiostatus = CDROM_AUDIO_PAUSED;
 		currently_playing = NULL;
@@ -750,6 +795,7 @@ bool CDResume_bincue(void *fh)
 	if (cs && player) {
 		// Pause another player if needed
 		CDPause_playing(player);
+		player->scanning = false;
 
 		// doesn't matter if it was paused, just ensure this one plays now
 		player->audiostatus = CDROM_AUDIO_PLAY;
@@ -759,31 +805,54 @@ bool CDResume_bincue(void *fh)
 	return false;
 }
 
+bool static PreparePlayOrScanAudio(CDPlayer *player) {
+	if (player->audio_enabled) {
+		player->audiostatus = CDROM_AUDIO_PLAY;
+#ifdef OSX_CORE_AUDIO
+		D(bug("starting os x sound\n"));
+		player->soundoutput.setCallback(bincue_core_audio_callback);
+		// should be from current track !
+		player->soundoutput.start(16, 2, 44100);
+#endif
+		currently_playing = player;
+		return true;
+	} else {
+		D(bug("play but player audio not enabled\n"));
+		return false;
+	}
+}
+
 bool CDPlay_bincue(void *fh, uint8 start_m, uint8 start_s, uint8 start_f,
 				   uint8 end_m, uint8 end_s, uint8 end_f)
 {
 	CueSheet *cs = (CueSheet *) fh;
 	CDPlayer *player = CSToPlayer(cs);
-	
+	bool result = false;
 	if (cs && player) {
+		LOCK_PLAYER;
+
 		// Pause another player if needed
 		CDPause_playing(player);
+		player->scanning = false;
 
 		int track;
 		MSF msf;
 
-#if defined(USE_SDL_AUDIO) && !SDL_VERSION_ATLEAST(3, 0, 0)
-		SDL_LockAudio();
-#endif
-
 		player->audiostatus = CDROM_AUDIO_NO_STATUS;
 
-		player->audiostart = (start_m * 60 * CD_FRAMES) +
-			(start_s * CD_FRAMES) + start_f;
-		player->audioend	= (end_m * 60 * CD_FRAMES) + (end_s * CD_FRAMES) + end_f;
+		int cur_position_frames = (player->audioposition / cs->raw_sector_size) + player->audiostart;
+
+		player->audiostart = MSFToFrames((MSF){start_m, start_s, start_f});
+		player->audioend   = MSFToFrames((MSF){end_m, end_s, end_f});
 
 		track = PositionToTrack(player->cs, player->audiostart);
 
+		int cur_track = PositionToTrack(player->cs, cur_position_frames);
+		MSF cur_msf;
+		FramesToMSF(cur_position_frames, &cur_msf);
+		D(bug("Track position check: requested play start %d m %d s %d f == track %d, current pos %d m %d s %d f == track %d\n",
+			start_m, start_s, start_f, track, cur_msf.m, cur_msf.s, cur_msf.f, cur_track));
+
 		if (track < player->cs->tcnt) {
 			player->audioposition = 0;
 
@@ -817,23 +886,14 @@ bool CDPlay_bincue(void *fh, uint8 start_m, uint8 start_s, uint8 start_f,
 		else
 			D(bug("CDPlay_bincue: play beyond last track !\n"));
 
-#if defined(USE_SDL_AUDIO) && !SDL_VERSION_ATLEAST(3, 0, 0)
-		SDL_UnlockAudio();
-#endif
-
-		if (player->audio_enabled) {
-			player->audiostatus = CDROM_AUDIO_PLAY;
-#ifdef OSX_CORE_AUDIO
-			D(bug("starting os x sound"));
-			player->soundoutput.setCallback(bincue_core_audio_callback);
-			// should be from current track !
-			player->soundoutput.start(16, 2, 44100);
-#endif
-			currently_playing = player;
-			return true;
+		if (cs->tracks[track].tcf != AUDIO) {
+			D(bug("CDPlay_bincue: not playing data track %d!\n", track));
+		} else if (PreparePlayOrScanAudio(player)) {
+			result = true;
 		}
+		UNLOCK_PLAYER;
 	}
-	return false;
+	return result;
 }
 
 bool CDScan_bincue(void *fh, uint8 start_m, uint8 start_s, uint8 start_f, bool reverse) {
@@ -841,23 +901,37 @@ bool CDScan_bincue(void *fh, uint8 start_m, uint8 start_s, uint8 start_f, bool r
 	CDPlayer *player = CSToPlayer(cs);
 	
 	if (cs && player) {
-		uint8 scanrate = 8; // 8x scan default but could use different value or make configurable
-		
-		MSF msf;
-		msf.m = start_m; msf.s = start_s; msf.f = start_f;
-		int current_frame = MSFToFrames(msf);
-		
-		if (reverse) {
-			msf.s -= scanrate;
-			int goto_frame = MSFToFrames(msf);
-			player->audioposition -= (current_frame - goto_frame) * player->cs->raw_sector_size;
+		int goto_frame = MSFToFrames((MSF){start_m, start_s, start_f});
+
+		int scan_starting_track = PositionToTrack(cs, goto_frame);
+		if (cs->tracks[scan_starting_track].tcf != AUDIO) {
+			D(bug(" scan starting from non-audio track\n"));
+			return false;
 		}
-		else {
-			msf.s += scanrate;
-			int goto_frame = MSFToFrames(msf);
-			player->audioposition += (goto_frame - current_frame) * player->cs->raw_sector_size;
+
+		/* Figure out the bounds of this audio region */
+		int first_audio_track = scan_starting_track;
+		int last_audio_track = scan_starting_track;
+
+		for (int track = scan_starting_track - 1; track >= 0               && cs->tracks[track].tcf == AUDIO; track--)
+			first_audio_track--;
+		for (int track = scan_starting_track + 1; track < player->cs->tcnt && cs->tracks[track].tcf == AUDIO; track++)
+			last_audio_track++;
+
+		player->audiostart = cs->tracks[first_audio_track].start;
+		player->fileoffset = cs->tracks[first_audio_track].fileoffset;
+		player->audioend = cs->tracks[last_audio_track].start + cs->tracks[last_audio_track].length;
+
+		player->silence = 0;
+
+		player->audioposition = (goto_frame - player->audiostart) * player->cs->raw_sector_size;
+		player->reverse = reverse;
+		player->scanning = true;
+
+
+		if (PreparePlayOrScanAudio(player)) {
+			return true;
 		}
-		return true;
 	}
     return false;
 }
@@ -908,43 +982,104 @@ static uint8 *fill_buffer(int stream_len, CDPlayer* player)
 	if (player->audiostatus == CDROM_AUDIO_PLAY) {
 		int remaining_silence = player->silence - player->audioposition;
 
-		if (player->audiostart + player->audioposition/player->cs->raw_sector_size
-			>= player->audioend) {
-			player->audiostatus = CDROM_AUDIO_COMPLETED;
-			return buf;
+		int current_read_bytes_limit = -1;
+		int full_read_bytes_limit = -1;
+		int jump_bytes_after = 0;
+
+		if (player->scanning) {
+			/* In a scan we alternate playing frames and jumping frames */
+
+			/* These values are from the "ATA Packet Interface for CD-ROMs"
+			   SCAN command's "Request to the implementer" */
+			int play_frames = 6;
+			int jump_frames = player->reverse? -150 : 190;
+
+			/* For testing, use some big values so you can tell what's going on: */
+			/*
+			int play_frames = 75*2;
+			int jump_frames = player->reverse? -75*5 : 75*5;
+			*/
+
+			/* Let's call one block of frames to play and one gap in the direction we're seeking a cycle */
+			int cycle_size_frames = std::abs(play_frames + jump_frames);
+
+			full_read_bytes_limit = play_frames * player->cs->raw_sector_size;
+			jump_bytes_after = jump_frames * player->cs->raw_sector_size;
+
+			/* Let's make the cycles aligned to audiostart for convenience.
+			   Handle where we are in the cycle */
+			int cycle_size_bytes = cycle_size_frames * player->cs->raw_sector_size;
+			int cycle_bytes_offset = player->audioposition % cycle_size_bytes;
+			if (cycle_bytes_offset < full_read_bytes_limit) {
+				// in a play block
+				current_read_bytes_limit = full_read_bytes_limit - cycle_bytes_offset;
+			} else {
+				// currently in a gap, move to the start of the next play block
+				int delta = - cycle_bytes_offset + full_read_bytes_limit + jump_bytes_after;
+				if ((int)player->audioposition + delta < 0) {
+					player->audiostatus = CDROM_AUDIO_COMPLETED;
+					return buf;
+				}
+				player->audioposition += delta;
+				current_read_bytes_limit = full_read_bytes_limit;
+			}
 		}
 
-		if (remaining_silence >= stream_len) {
-			player->audioposition += stream_len;
-			return buf;
-		}
+		int available;
+		do {
+			if (player->audiostart + player->audioposition/player->cs->raw_sector_size >= player->audioend) {
+				player->audiostatus = CDROM_AUDIO_COMPLETED;
+				return buf;
+			}
 
-		if (remaining_silence > 0) {
-			offset += remaining_silence;
-			player->audioposition += remaining_silence;
-		}
+			if (remaining_silence >= stream_len) {
+				player->audioposition += stream_len;
+				return buf;
+			}
 
-		int available = ((player->audioend - player->audiostart) *
-						 player->cs->raw_sector_size) - player->audioposition;
-		if (available > (stream_len - offset))
-			available = stream_len - offset;
+			if (remaining_silence > 0) {
+				offset += remaining_silence;
+				player->audioposition += remaining_silence;
+			}
 
-		if (lseek(player->audiofh,
-				  player->fileoffset + player->audioposition - player->silence,
-					  SEEK_SET) < 0)
-			return NULL;
+			available = ((player->audioend - player->audiostart) *
+						player->cs->raw_sector_size) - player->audioposition;
+			if (available > (stream_len - offset))
+				available = stream_len - offset;
 
-		if (available < 0) {
-			player->audioposition += available; // correct end !;
-			available = 0;
-		}
+			bool hit_read_limit = false;
+			if (current_read_bytes_limit != -1) {
+				if (available >= current_read_bytes_limit) {
+					available = current_read_bytes_limit;
+					hit_read_limit = true;
+				}
+			}
+			current_read_bytes_limit = full_read_bytes_limit;
 
-		ssize_t ret = 0;
-		if ((ret = read(player->audiofh, &buf[offset], available)) >= 0) {
-			player->audioposition += ret;
-			offset += ret;
-			available -= ret;
-		}
+			if (lseek(player->audiofh,
+					  player->fileoffset + player->audioposition - player->silence,
+						  SEEK_SET) < 0)
+				return NULL;
+
+			if (available < 0) {
+				player->audioposition += available; // correct end !;
+				available = 0;
+			}
+
+			ssize_t ret = 0;
+			if ((ret = read(player->audiofh, &buf[offset], available)) >= 0) {
+				player->audioposition += ret;
+				offset += ret;
+				available -= ret;
+			}
+
+			if ((int)player->audioposition + jump_bytes_after < 0) {
+				player->audiostatus = CDROM_AUDIO_COMPLETED;
+				return buf;
+			}
+			if (hit_read_limit)
+				player->audioposition += jump_bytes_after;
+		} while (player->scanning && offset < stream_len);
 
 		while (offset < stream_len) {
 			buf[offset++] = silence_byte;
@@ -958,64 +1093,135 @@ static uint8 *fill_buffer(int stream_len, CDPlayer* player)
 
 
 #ifdef USE_SDL_AUDIO
-void MixAudio_bincue(uint8 *stream, int stream_len, int volume)
+
+bool HaveAudioToMix_bincue() {
+	return currently_playing != NULL;
+}
+
+void MixAudio_bincue(uint8 *stream, int dest_stream_len)
 {
+	if (!dest_stream_len) return;
+
 	if (currently_playing) {
-		
+		LOCK_PLAYER;
+
 		CDPlayer *player = currently_playing;
-		
+
+		OutputSettings & o = current_output_settings;
+
+		/** How many bytes do we need in terms of source data (CD audio)? */
+		int source_channels_sample = 44100 * 2 * 2;
+#if SDL_VERSION_ATLEAST(3, 0, 0)
+		int dest_format_bytes = SDL_AUDIO_BYTESIZE((SDL_AudioFormat) o.format);
+#else
+		int dest_format_bytes = o.format == AUDIO_U8 ? 1 : 2;
+#endif
+		int dest_channels_sample = o.freq * o.channels * dest_format_bytes;
+		int src_stream_len = (int)((uint64) dest_stream_len * source_channels_sample / dest_channels_sample);
+
 		if (player->audiostatus == CDROM_AUDIO_PLAY) {
-			uint8 *buf = fill_buffer(stream_len, player);
+			//D(bug("MixAudio cd playing, player=0x%p\n", player));
+			uint8 *buf = fill_buffer(src_stream_len, player);
 #if SDL_VERSION_ATLEAST(3, 0, 0)
 			if (buf)
-				SDL_PutAudioStreamData(player->stream, buf, stream_len);
+				SDL_PutAudioStreamData(player->stream, buf, src_stream_len);
 			int avail = SDL_GetAudioStreamAvailable(player->stream);
-			if (avail >= stream_len) {
-				extern SDL_AudioSpec audio_spec;
-				uint8 converted[stream_len];
-				SDL_GetAudioStreamData(player->stream, converted, stream_len);
-				SDL_MixAudioFormat(stream, converted, audio_spec.format, stream_len, player->volume_mono);
+			if (avail >= dest_stream_len) {
+				//D(bug("have bytes avail %d stream len %d\n", avail, dest_stream_len));
+				uint8 converted[dest_stream_len];
+				SDL_GetAudioStreamData(player->stream, converted, dest_stream_len);
+				float volume = (float)player->volume_mono/128;
+				// Apply 60% volume while scanning (ff/reverse)
+				if (player->scanning) volume *= 0.6;
+				SDL_MixAudio(stream, converted, (SDL_AudioFormat) o.format, dest_stream_len, volume);
 			}
 #else
 			if (buf)
-				SDL_AudioStreamPut(player->stream, buf, stream_len);
+				SDL_AudioStreamPut(player->stream, buf, src_stream_len);
 			int avail = SDL_AudioStreamAvailable(player->stream);
-			if (avail >= stream_len) {
-				uint8 converted[stream_len];
-				SDL_AudioStreamGet(player->stream, converted, stream_len);
-				SDL_MixAudio(stream, converted, stream_len, player->volume_mono);
+			if (avail >= dest_stream_len) {
+				//D(bug("have bytes avail %d stream len %d\n", avail, dest_stream_len));
+				uint8 converted[dest_stream_len];
+				SDL_AudioStreamGet(player->stream, converted, dest_stream_len);
+				int volume = player->volume_mono;
+				// Apply 60% volume while scanning (ff/reverse)
+				if (player->scanning) volume = volume * 3 / 5;
+				SDL_MixAudio(stream, converted, dest_stream_len, volume);
 			}
 #endif
 		}
-		
+		UNLOCK_PLAYER;
 	}
 }
 
+static void OpenPlayerStream(CDPlayer * player) {
+	if (!have_current_output_settings) {
+		player->stream = NULL;
+		return;
+	}
+	OutputSettings & o = current_output_settings;
+
+	// set player volume based on SDL volume
+	player->volume_left = player->volume_right = player->volume_mono = o.default_cd_player_volume;
+	// audio stream handles converting cd audio to destination output
+	D(bug("Opening player stream\n"))
+#if SDL_VERSION_ATLEAST(3, 0, 0)
+	SDL_AudioSpec src = { player->cs->big_endian_audio? SDL_AUDIO_S16BE : SDL_AUDIO_S16LE, 2, 44100 };
+	SDL_AudioSpec dst = { (SDL_AudioFormat)o.format, o.channels, o.freq };
+	player->stream = SDL_CreateAudioStream(&src, &dst);
+#else
+	player->stream = SDL_NewAudioStream(player->cs->big_endian_audio? AUDIO_S16MSB : AUDIO_S16LSB, 2, 44100,
+										o.format, o.channels, o.freq);
+#endif
+	if (player->stream == NULL) {
+		D(bug("Failed to open CD player audio stream using SDL!\n"));
+	}
+	else {
+		player->audio_enabled = true;
+	}
+}
+
+static void ClosePlayerStream(CDPlayer * player)
+{
+#if !SDL_VERSION_ATLEAST(3, 0, 0)
+#define SDL_DestroyAudioStream	SDL_FreeAudioStream
+#endif
+	if (player->stream) // if audiostream has been opened, free it as well
+		SDL_DestroyAudioStream(player->stream);
+	player->stream = NULL;
+}
+
 void OpenAudio_bincue(int freq, int format, int channels, uint8 silence, int volume)
 {
+	// save output audio params
+	current_output_settings = (OutputSettings){freq, format, channels, volume};
+	have_current_output_settings = true;
+#if SDL_VERSION_ATLEAST(3, 0, 0)
+	D(bug("OpenAudio_bincue freq %d format %s channels %d volume %d\n",
+		current_output_settings.freq, SDL_GetAudioFormatName((SDL_AudioFormat)current_output_settings.format),
+		current_output_settings.channels, current_output_settings.default_cd_player_volume));
+#else
+	D(bug("OpenAudio_bincue freq %d format %d channels %d volume %d\n",
+		current_output_settings.freq, current_output_settings.format,
+		current_output_settings.channels, current_output_settings.default_cd_player_volume));
+#endif
 	// setup silence at init
 	silence_byte = silence;
-	
-	// init players
-	for (std::vector<CDPlayer*>::iterator it = players.begin(); it != players.end(); ++it)
+
+	// init players for these settings
+	for (std::list<CDPlayer*>::iterator it = players.begin(); it != players.end(); ++it)
 	{
 		CDPlayer *player = *it;
-		
-		// set player volume based on SDL volume
-		player->volume_left = player->volume_right = player->volume_mono = volume;
-		// audio stream handles converting cd audio to destination output
-#if SDL_VERSION_ATLEAST(3, 0, 0)
-		SDL_AudioSpec src = { SDL_AUDIO_S16LE, 2, 44100 }, dst = { (SDL_AudioFormat)format, channels, freq };
-		player->stream = SDL_CreateAudioStream(&src, &dst);
-#else
-		player->stream = SDL_NewAudioStream(AUDIO_S16LSB, 2, 44100, format, channels, freq);
-#endif
-		if (player->stream == NULL) {
-			D(bug("Failed to open CD player audio stream using SDL!"));
-		}
-		else {
-			player->audio_enabled = true;
-		}
+		OpenPlayerStream(player);
+	}
+}
+
+void CloseAudio_bincue() {
+	have_current_output_settings = false;
+	for (std::list<CDPlayer*>::iterator it = players.begin(); it != players.end(); ++it)
+	{
+		CDPlayer *player = *it;
+		ClosePlayerStream(player);
 	}
 }
 #endif
@@ -1023,7 +1229,7 @@ void OpenAudio_bincue(int freq, int format, int channels, uint8 silence, int vol
 #ifdef OSX_CORE_AUDIO
 static int bincue_core_audio_callback(void)
 {
-	for (std::vector<CDPlayer*>::iterator it = players.begin(); it != players.end(); ++it)
+	for (std::list<CDPlayer*>::iterator it = players.begin(); it != players.end(); ++it)
 	{
 		CDPlayer *player = *it;
 		
