@@ -39,6 +39,12 @@ enum {
 	STR_START_BUTTON,
 	STR_QUIT_BUTTON,
 	STR_CANCEL_BUTTON,
+	STR_CONTINUE_BUTTON,
+	STR_SELECT_BUTTON,
+	STR_CREATE_BUTTON,
+	STR_APP_NAME,
+	STR_APP_DISPLAY_NAME,
+	STR_APP_ID,
 
 	// Error messages
 	STR_NO_MEM_ERR = 1000,
@@ -99,6 +105,9 @@ enum {
 	STR_VOL_SIZE_CTRL,
 	STR_VOL_BLOCKSIZE_CTRL,
 	STR_VOL_FILE_CTRL,
+	STR_VOL_HEADING_LOCATION,
+	STR_VOL_HEADING_CDROM,
+	STR_VOL_HEADING_SIZE,
 
 	STR_SCSI_PANE_TITLE = 3300,		// SCSI pane
 	STR_SCSI_ID_0 = 3301,
@@ -168,6 +177,10 @@ enum {
 	STR_DIRECT3D_LAB,
 	STR_GRAPHICS_SDL_VSYNC_CTRL,
 	STR_DEFAULT_LAB,
+	STR_SDL_SCALING,
+	STR_SCALE_FACTOR,
+	STR_SCALE_NEAREST,
+	STR_SCALE_INTEGER,
 
 	STR_SERIAL_NETWORK_PANE_TITLE = 3500,	// Serial/Networking pane
 	STR_SERIALA_CTRL,
