@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <SDL.h>
+#include "my_sdl.h"
 
 #include "sysdeps.h"
 #include "main.h"
@@ -83,6 +83,9 @@ uint8 *ROMBaseHost;		// Base address of Mac ROM (host address space)
 DWORD win_os;			// Windows OS id
 DWORD win_os_major;		// Windows OS version major
 
+#ifdef MEM_BULK
+uint8 gKernelData[0x2000];
+#endif
 
 // Global variables
 static int kernel_area = -1;				// SHM ID of Kernel Data area
@@ -105,7 +108,7 @@ static uintptr sig_stack = 0;				// Stack for PowerPC interrupt routine
 
 uint32  SheepMem::page_size;				// Size of a native page
 uintptr SheepMem::zero_page = 0;			// Address of ro page filled in with zeros
-uintptr SheepMem::base = 0x60000000;		// Address of SheepShaver data
+uintptr SheepMem::base = 0x51000000;		// Address of SheepShaver data
 uintptr SheepMem::proc;						// Bottom address of SheepShave procedures
 uintptr SheepMem::data;						// Top of SheepShaver data (stack like storage)
 
@@ -184,6 +187,8 @@ int main(int argc, char **argv)
 	printf(" %s\n", GetString(STR_ABOUT_TEXT2));
 
 	// Parse command line arguments
+
+	// Check for options we want to process before PrefsInit
 	for (int i=1; i<argc; i++) {
 		if (strcmp(argv[i], "--help") == 0) {
 			usage(argv[0]);
@@ -194,15 +199,33 @@ int main(int argc, char **argv)
 				UserPrefsPath = to_tstring(argv[i]);
 				argv[i] = NULL;
 			}
-		} else if (argv[i][0] == '-') {
-			fprintf(stderr, "Unrecognized option '%s'\n", argv[i]);
-			usage(argv[0]);
+		}
+	}
+
+	// Remove processed arguments
+	for (int i=1; i<argc; i++) {
+		int k;
+		for (k=i; k<argc; k++)
+			if (argv[k] != NULL)
+				break;
+		if (k > i) {
+			k -= i;
+			for (int j=i+k; j<argc; j++)
+				argv[j-k] = argv[j];
+			argc -= k;
 		}
 	}
 
 	// Read preferences
 	PrefsInit(NULL, argc, argv);
 
+	for (int i=1; i<argc; i++) {
+		if (argv[i][0] == '-') {
+			fprintf(stderr, "Unrecognized option '%s'\n", argv[i]);
+			usage(argv[0]);
+		}
+	}
+
 	// #chenchijung 2024/2/21: move vm_init(), memory allocation for Mac RAM and Mac ROM here to avoid "cannot map RAM: no Error" bug.
 	//   caused by MSI afterburner (RIVA Tuner statistic tuner Server?). It is a workaround since I don't know why. But it works in my test env.
 	//
@@ -324,15 +347,17 @@ int main(int argc, char **argv)
 		if (!PrefsEditor())
 			goto quit;
 
+#ifndef MEM_BULK
 	// Create areas for Kernel Data
 	if (!kernel_data_init())
 		goto quit;
+#endif
 	kernel_data = (KernelData *)Mac2HostAddr(KERNEL_DATA_BASE);
 	emulator_data = &kernel_data->ed;
 	KernelDataAddr = KERNEL_DATA_BASE;
 	D(bug("Kernel Data at %p (%08x)\n", kernel_data, KERNEL_DATA_BASE));
 	D(bug("Emulator Data at %p (%08x)\n", emulator_data, KERNEL_DATA_BASE + offsetof(KernelData, ed)));
-
+#if 0
 	// Create area for DR Cache
 	if (vm_mac_acquire(DR_EMULATOR_BASE, DR_EMULATOR_SIZE) < 0) {
 		sprintf(str, GetString(STR_DR_EMULATOR_MMAP_ERR), strerror(errno));
@@ -348,7 +373,7 @@ int main(int argc, char **argv)
 	dr_cache_area_mapped = true;
 	DRCacheAddr = (uint32)Mac2HostAddr(DR_CACHE_BASE);
 	D(bug("DR Cache at %p (%08x)\n", DRCacheAddr, DR_CACHE_BASE));
-
+#endif
 	// Create area for SheepShaver data
 	if (!SheepMem::Init()) {
 		sprintf(str, GetString(STR_SHEEP_MEM_MMAP_ERR), strerror(errno));
@@ -801,9 +826,7 @@ void SheepMem::Exit(void)
  */
 
 #ifdef USE_SDL_VIDEO
-#if SDL_VERSION_ATLEAST(3, 0, 0)
-#include <SDL_video.h>
-#else
+#if !SDL_VERSION_ATLEAST(3, 0, 0)
 #include <SDL_syswm.h>
 #endif
 extern SDL_Window *sdl_window;
@@ -814,7 +837,7 @@ HWND GetMainWindowHandle(void)
 	}
 #if SDL_VERSION_ATLEAST(3, 0, 0)
 	SDL_PropertiesID props = SDL_GetWindowProperties(sdl_window);
-	return (HWND)SDL_GetProperty(props, "SDL.window.cocoa.window", NULL);
+	return (HWND)SDL_GetPointerProperty(props, "SDL.window.cocoa.window", NULL);
 #else
 	SDL_SysWMinfo wmInfo;
 	SDL_VERSION(&wmInfo.version);
@@ -899,8 +922,13 @@ static LRESULT CALLBACK low_level_keyboard_hook(int nCode, WPARAM wParam, LPARAM
 					SDL_Event e;
 					memset(&e, 0, sizeof(e));
 					e.type = (wParam == WM_KEYDOWN) ? SDL_EVENT_KEY_DOWN : SDL_EVENT_KEY_UP;
+#if SDL_VERSION_ATLEAST(3, 0, 0)
+					e.key.key = (p->vkCode == VK_LWIN) ? SDLK_LGUI : SDLK_RGUI;
+					e.key.scancode = (p->vkCode == VK_LWIN) ? SDL_SCANCODE_LGUI : SDL_SCANCODE_RGUI;
+#else
 					e.key.keysym.sym = (p->vkCode == VK_LWIN) ? SDLK_LGUI : SDLK_RGUI;
 					e.key.keysym.scancode = (p->vkCode == VK_LWIN) ? SDL_SCANCODE_LGUI : SDL_SCANCODE_RGUI;
+#endif
 					SDL_PushEvent(&e);
 					return 1;
 				}
