@@ -1,7 +1,7 @@
 /*
  *  prefs_editor.h - Preferences editor
  *
- *  SheepShaver (C) 1997-2008 Christian Bauer and Marc Hellwig
+ *  Basilisk II (C) 1997-2008 Christian Bauer
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,4 +27,21 @@ extern void PrefsEditor(uint32 msg);
 extern bool PrefsEditor(void);
 #endif
 
+#if defined(ENABLE_GTK) || defined(STANDALONE_GUI)
+
+#include <gtk/gtk.h>
+
+#if !GLIB_CHECK_VERSION(2, 24, 0)
+#define GVariant void
+#endif
+#if !GLIB_CHECK_VERSION(2, 28, 0)
+#define GSimpleAction void
+#endif
+
+extern "C" {
+void dl_quit(GtkWidget *dialog);
+void cb_swap_opt_cmd (GtkWidget *widget);
+void cb_infobar_show (GtkWidget *widget);
+}
+#endif
 #endif
