@@ -40,6 +40,12 @@ enum {
 	STR_QUIT_BUTTON,
 	STR_CANCEL_BUTTON,
 	STR_IGNORE_BUTTON,
+	STR_CONTINUE_BUTTON,
+	STR_SELECT_BUTTON,
+	STR_CREATE_BUTTON,
+	STR_APP_NAME,
+	STR_APP_DISPLAY_NAME,
+	STR_APP_ID,
 
 	// Error messages
 	STR_NOT_ENOUGH_MEMORY_ERR = 1000,
@@ -92,6 +98,9 @@ enum {
 	STR_ADD_VOLUME_TITLE,
 	STR_CREATE_VOLUME_TITLE,
 	STR_HARDFILE_SIZE_CTRL,
+	STR_VOL_HEADING_LOCATION,
+	STR_VOL_HEADING_CDROM,
+	STR_VOL_HEADING_SIZE,
 
 	// Graphics pane
 	STR_GRAPHICS_SOUND_PANE_TITLE = 3300,
@@ -138,6 +147,10 @@ enum {
 	STR_DIRECT3D_LAB,
 	STR_GRAPHICS_SDL_VSYNC_CTRL,
 	STR_DEFAULT_LAB,
+	STR_SDL_SCALING,
+	STR_SCALE_FACTOR,
+	STR_SCALE_NEAREST,
+	STR_SCALE_INTEGER,
 
 	// Serial/Network pane
 	STR_SERIAL_NETWORK_PANE_TITLE = 3400,
@@ -163,9 +176,9 @@ enum {
 	STR_IGNORESEGV_CTRL,
 	STR_IDLEWAIT_CTRL,
 	STR_ROM_FILE_CTRL,
-  STR_NOGUI_CTRL,
-  STR_NOGUI_TIP,
-  STR_NOGUI_TIP2,
+	STR_NOGUI_CTRL,
+	STR_NOGUI_TIP,
+	STR_NOGUI_TIP2,
 
 	// JIT Compiler pane
 	STR_JIT_PANE_TITLE = 3700,
