@@ -58,6 +58,12 @@ user_string_def common_strings[] = {
 	{STR_QUIT_BUTTON, "Quit"},
 	{STR_CANCEL_BUTTON, "Cancel"},
 	{STR_IGNORE_BUTTON, "Ignore"},
+	{STR_CONTINUE_BUTTON, "Continue"},
+	{STR_SELECT_BUTTON, "Select"},
+	{STR_CREATE_BUTTON, "Create"},
+	{STR_APP_NAME, "SheepShaver"},
+	{STR_APP_DISPLAY_NAME, "SheepShaver"},
+	{STR_APP_ID, "net.cebix.SheepShaver"},
 
 	{STR_NOT_ENOUGH_MEMORY_ERR, "Your computer does not have enough memory to run SheepShaver."},
 	{STR_NO_KERNEL_DATA_ERR, "Cannot create Kernel Data area: %s (%08x)."},
@@ -93,6 +99,9 @@ user_string_def common_strings[] = {
 	{STR_ADD_VOLUME_BUTTON, "Add" ELLIPSIS},
 	{STR_CREATE_VOLUME_BUTTON, "Create" ELLIPSIS},
 	{STR_REMOVE_VOLUME_BUTTON, "Remove"},
+	{STR_VOL_HEADING_LOCATION, "Location"},
+	{STR_VOL_HEADING_CDROM, "CD-ROM"},
+	{STR_VOL_HEADING_SIZE, "Size"},
 	{STR_ADD_VOLUME_PANEL_BUTTON, "Add"},
 	{STR_CREATE_VOLUME_PANEL_BUTTON, "Create"},
 	{STR_CDROM_DRIVE_CTRL, "CD-ROM Drive"},
@@ -148,6 +157,10 @@ user_string_def common_strings[] = {
 	{STR_DIRECT3D_LAB, "Direct3D"},
 	{STR_GRAPHICS_SDL_VSYNC_CTRL, "Vertical Sync (Software)"},
 	{STR_DEFAULT_LAB, "Default"},
+	{STR_SDL_SCALING, "Scaling (SDL)"},
+	{STR_SCALE_FACTOR, "Scale factor:"},
+	{STR_SCALE_NEAREST, "Nearest"},
+	{STR_SCALE_INTEGER, "Integer Scaling"},
 
 	{STR_SERIAL_NETWORK_PANE_TITLE, "Serial/Network"},
 	{STR_SERPORTA_CTRL, "Modem Port"},
